// SPDX-FileCopyrightText: 2025 Tuxilio <mail@tuxil.io>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

const SYNC_INTERVAL = 30000;

document.addEventListener("DOMContentLoaded", function() {
    //const parameters = new URLSearchParams(window.location.search);

    const ui = document.getElementById('ui');
    const loader = document.getElementById("loader");
    const loadingScreen = document.getElementById("loading-screen");
    const mainUI = document.getElementById("main-ui");

    let baseUrl;
    let matrixServer;
    let matrixID;
    let matrixToken;

    let filterId;

    // Obvious //
    function setCookie(cname, cvalue) {
        const exdays = 7;
        const d = new Date();
        d.setTime(d.getTime() + (exdays * 24 * 60 * 60 * 1000));
        let expires = "expires="+d.toUTCString();
        document.cookie = cname + "=" + cvalue + ";" + expires + ";path=/;SameSite=Strict";
    }

    function getCookie(cname) {
        let name = cname + "=";
        let ca = document.cookie.split(';');
        for(let i = 0; i < ca.length; i++) {
            let c = ca[i];
            while (c.charAt(0) == ' ') {
                c = c.substring(1);
            }
            if (c.indexOf(name) == 0) {
                return c.substring(name.length, c.length);
            }
        }
        return "";
    }

    // Main //
    async function ui_loading() {
        loadingScreen.style.display = 'flex';
        mainUI.style.display = 'none';
    }

    async function ui_main() {
        loadingScreen.style.display = 'none';
        mainUI.style.display = 'flex';
    }

    // UI intercation //

    async function connect() {
        await loginWindow();
    }

    async function loginWindow() {
        console.log('Showing login window');
        ui.innerHTML = `
            <h3>Login</h3>
            <input type="text" id="matrix_server" name="matrix_server" placeholder="Matrix-Server">
            <br>
            <input type="text" id="matrix_id" name="matrix_id" placeholder="Matrix ID">
            <br>
            <input type="password" id="matrix_token" name="matrix_token" placeholder="Access Token">
            <br>
            <input type="checkbox" id="skip_login_window" name="skip_login_window" value="Skip Login Window">
            <label for="skip_login_window"> Skip login window next time</label>
            <br>
            <button type="button" class="next-button" onclick="login()">
                Next
                <span class="arrow">→</span>
            </button>
        `;

        document.getElementById('matrix_server').value = await getCookie("matrixServer");
        document.getElementById('matrix_id').value = await getCookie("matrixId");
        document.getElementById('matrix_token').value = await getCookie("matrixToken");

        const skipLoginWindow = await getCookie("skipLoginWindow");
        document.getElementById('skip_login_window').checked = skipLoginWindow;

        await ui_main()

        if (skipLoginWindow === "true") {
            login()
        }
    }

    window.login = async function() {
        matrixServer = document.getElementById('matrix_server').value;
        matrixId = document.getElementById('matrix_id').value;
        matrixToken = document.getElementById('matrix_token').value;

        skipLoginWindow = document.getElementById('skip_login_window').checked;

        setCookie("matrixServer", matrixServer);
        setCookie("matrixId", matrixId);
        setCookie("matrixToken", matrixToken);

        setCookie("skipLoginWindow", skipLoginWindow);

        if (!matrixServer || !matrixId || !matrixToken) {
            alert("Please enter Matrix server, ID and Access Token!");
            return;
        }

        await ui_loading()

        baseUrl = `${matrixServer}/_matrix/client/v3`;

        try {
            await userInfo();
            await createFilter()
            await discoverSessions();
        } catch (error) {
            console.error('Login failed:', error);
            alert("Login failed. Please check your credentials.");
        } finally {
            await ui_main()
        }
    }

    async function userInfo() {
        console.log("Fetching user info...");
        try {
            const response = await fetch(`${baseUrl}/profile/${matrixId}`, {
                method: 'GET',
                headers: {
                    'Authorization': `Bearer ${matrixToken}`,
                    'Content-Type': 'application/json'
                }
            });

            if (!response.ok) {
                throw new Error(`Error: ${response.status} ${response.statusText}`);
            }

            const data = await response.json();

            if (data.avatar_url) {
                const avatarResponse = await fetch(`${matrixServer}/_matrix/media/r0/download/${data.avatar_url.split('/')[2]}/${data.avatar_url.split('/')[3]}`);
                const avatarBlob = await avatarResponse.blob();
                const avatarUrl = URL.createObjectURL(avatarBlob);
                document.getElementById('avatar').src = avatarUrl;
            }

            document.getElementById('username').textContent = data.displayname || 'Unknown';
            document.getElementById('user-info').style.display = 'flex';
        } catch (error) {
            console.error('Failed to fetch user info:', error);
            throw error;
        }
    }

    async function createFilter() {
        console.log("Creating filter...");

        const response = await fetch(`${baseUrl}/user/${matrixId}/filter`, {
            method: 'POST',
            headers: {
                'Authorization': `Bearer ${matrixToken}`,
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                "account_data": {
                    "not_types": ["*"]
                },
                "presence": {
                    "not_types": ["*"]
                },
                "room": {
                    "ephemeral": {
                        "not_types": ["*"]
                    },
                    "state": {
                        "lazy_load_members": true,
                        "types": ["org.edugit.auglement.festrix.*"]
                    },
                    "timeline": {
                        "lazy_load_members": true,
                        "not_types": ["*"]
                    }
                }
            })
        });

        const data = await response.json();
        filterId = data.filter_id;
        console.log(`  --> Filter id: ${filterId}`);
    }

    async function getRoomName(roomId) {
        const roomNameResponse = await fetch(`${baseUrl}/rooms/${roomId}/state/m.room.name`, {
            method: 'GET',
            headers: {
                'Authorization': `Bearer ${matrixToken}`,
                'Content-Type': 'application/json'
            }
        });
        const roomNameJson = await roomNameResponse.json();
        const roomName = await roomNameJson.name;
        return roomName;
    }

    async function discoverSessions() {
        console.log("Discovering sessions...");

        await ui_loading()

        ui.innerHTML = "";

        // Public rooms
        const publicResponse = await fetch(`${baseUrl}/publicRooms`, {
            method: 'POST',
            headers: {
                'Authorization': `Bearer ${matrixToken}`,
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                filter: { room_types: ["org.edugit.auglement.festrix.session"] }
            })
        });

        const publicData = await publicResponse.json();
        console.log("  --> Public Rooms:", JSON.stringify(publicData, null, 2));

        if (publicData.chunk.length > 0) {
            publicData.chunk.forEach(room => {
                const button = document.createElement('button');
                button.className = 'session-button next-button';
                button.textContent = room.name || 'Unnamed room';

                const arrow = document.createElement('span');
                arrow.className = 'arrow';
                arrow.textContent = '→';
                button.appendChild(arrow);

                button.onclick = () => {
                    console.log(`Room ${room.name} selected.`);
                    selectRoom(room, room.name);
                };
                ui.appendChild(button);
            });
        }

        // Private rooms
        const privateResponse = await fetch(`${baseUrl}/sync?filter=${filterId}&set_presence=offline`, {
            method: 'GET',
            headers: {
                'Authorization': `Bearer ${matrixToken}`,
                'Content-Type': 'application/json'
            }
        });

        const privateData = await privateResponse.json();
        const privateRooms = privateData.rooms.join;

        const festrixPrivateRooms = Object.keys(privateRooms).filter(roomId => {
            const room = privateRooms[roomId];
            return room.state.events.some(event =>
                event.type.startsWith("org.edugit.auglement.festrix.")
            );
        });

        console.log("  --> Private Rooms:", JSON.stringify(festrixPrivateRooms, null, 2));

        if (festrixPrivateRooms.length > 0) {
            for (const roomId of festrixPrivateRooms) {
                const room = privateRooms[roomId];
                const button = document.createElement('button');
                button.className = 'session-button next-button';
                const roomName = await getRoomName(roomId);
                button.textContent = roomName || 'Unnamed private room';

                const arrow = document.createElement('span');
                arrow.className = 'arrow';
                arrow.textContent = '→';
                button.appendChild(arrow);

                button.onclick = () => {
                    selectRoom(roomId, roomName);
                };
                ui.appendChild(button);
            }
        }

        const createRoomButton = document.createElement('button');
        createRoomButton.className = 'session-button next-button';
        createRoomButton.textContent = 'New room';

        const createRoomButtonArrow = document.createElement('span');
        createRoomButtonArrow.className = 'arrow';
        createRoomButtonArrow.textContent = '→';

        createRoomButton.appendChild(createRoomButtonArrow);

        createRoomButton.onclick = () => {
            ui.innerHTML = `
                <h3>Create a new Festrix room</h3>
                <input type="text" id="roomname" name="roomname">
                <br>
                <input type="radio" name="matrix_createroom" value="private" id="matrix_createroom_private" checked>Private room</input>
                <input type="radio" name="matrix_createroom" value="public" id="matrix_createroom_public">Public room</input>
                <br>
                <button type="button" class="next-button" onclick="createRoom()">
                    Next
                    <span class="arrow">→</span>
                </button>
            `;
        };
        ui.appendChild(createRoomButton);
    }

    window.createRoom = async function() {
        await ui_loading()

        console.log("Creating new room...");

        let roomPreset;
        const name = document.getElementById('roomname').value;
        const roomPresetCheckbox = document.getElementById("matrix_createroom_public").checked;

        if (roomPresetCheckbox === "true") {
            roomPreset = "public_chat";
        } else {
            roomPreset = "private_chat";
        }

        const response = await fetch(`${baseUrl}/createRoom`, {
            method: 'POST',
            headers: {
                'Authorization': `Bearer ${matrixToken}`,
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                name: name,
                preset: roomPreset,
                creation_content: {
                    type: "org.edugit.auglement.festrix.session"
                },
                initial_state: [
                    {
                        type: "org.edugit.auglement.festrix.session",
                        state_key: "",
                        content: {
                            backends: []
                        }
                    },
                    {
                        type: "org.edugit.auglement.festrix.playlist",
                        state_key: "",
                        content: {
                            queue: []
                        }
                    }
                ],
                power_level_content_override: {
                    events: {
                        "org.edugit.auglement.festrix.session": 49,
                        "org.edugit.auglement.festrix.playlist": 49,
                        "org.edugit.auglement.festrix.vote": 0
                    }
                }
            })
        });

        if (!response.ok) {
            console.error('Failed to create room:', response.statusText);
            return;
        }

        const roomData = await response.json();
        console.log('Room created:', roomData);

        await discoverSessions()
        await await ui_main()
    }

    window.selectRoom = async function(roomId, roomName) {
        await ui_loading();

        console.log(`Room with ID "${roomId}", name "${roomName}" selected`);

        const powerLevels = await fetchPowerLevels(roomId);
        const powerLevel = powerLevels.users;
    }

    async function fetchPowerLevels(roomId) {
        const response = await fetch(`${baseUrl}/rooms/${roomId}/state/m.room.power_levels`, {
            method: 'GET',
            headers: {
                'Authorization': `Bearer ${matrixToken}`,
                'Content-Type': 'application/json'
            }
        });

        const data = await response.json();
        return data;
    }

    async function sync() {
        if (!filterId) {
            await createFilter();
        }

        const response = await fetch(`${baseUrl}/sync?filter=${filterId}&set_presence=offline`, {
            headers: {
                'Authorization': `Bearer ${matrixToken}`
            }
        });

        const data = await response.json();
        document.getElementById('ui').innerText = JSON.stringify(data, null, 2);
    }

    connect();
    // setInterval(sync, SYNC_INTERVAL);
});
